@@ -4,7 +4,7 @@
 """Init file"""
 from .read_db import db_reader
 from .water_eos import iapws95, ZhangDuan, water_dielec, readIAPWS95data, convert_temperature
-from .species_eos import heatcap, heatcapusgscal, supcrtaq, heatcap_Berman, Element_counts
+from .species_eos import heatcap, supcrtaq, Element_counts
 from .solid_solution import solidsolution_thermo
 from .clay_thermocalc import calclogKclays, MW
 from .pygcc_utils import *
